@@ -2,6 +2,7 @@
 
 use super::bin::{BinInfo, BinLimits, BinRemapper};
 use super::empty_subgrid::EmptySubgridV1;
+use super::evolution::{self, OperatorInfo};
 use super::fk_table::FkTable;
 use super::import_only_subgrid::ImportOnlySubgridV2;
 use super::lagrange_subgrid::{LagrangeSparseSubgridV1, LagrangeSubgridV1, LagrangeSubgridV2};
@@ -263,6 +264,9 @@ pub enum GridError {
         /// Maximum supported file format version for this library.
         supported_version: u64,
     },
+    /// Returned from [`Grid::evolve`] if the evolution failed.
+    #[error("failed to evolve grid: {0}")]
+    EvolutionFailure(String),
 }
 
 #[derive(Clone, Deserialize, Serialize)]
@@ -1301,31 +1305,9 @@ impl Grid {
     /// TODO
     #[must_use]
     pub fn axes(&self) -> Option<GridAxes> {
-        // determine what and how many hadrons are in the initial state
-        let initial_state_1 = self
-            .key_values()
-            .map_or(Some("2212"), |kv| {
-                kv.get("initial_state_1").map(String::as_str)
-            })
-            .map(str::parse::<i32>)?
-            .ok()?;
-        let initial_state_2 = self
-            .key_values()
-            .map_or(Some("2212"), |kv| {
-                kv.get("initial_state_2").map(String::as_str)
-            })
-            .map(str::parse::<i32>)?
-            .ok()?;
-
         // are the initial states hadrons?
-        let has_pdf1 = !self
-            .lumi()
-            .iter()
-            .all(|entry| entry.entry().iter().all(|&(a, _, _)| a == initial_state_1));
-        let has_pdf2 = !self
-            .lumi()
-            .iter()
-            .all(|entry| entry.entry().iter().all(|&(_, b, _)| b == initial_state_2));
+        let has_pdf1 = self.has_pdf1();
+        let has_pdf2 = self.has_pdf1();
 
         let mut mur2_grid = Vec::new();
         let mut muf2_grid = Vec::new();
@@ -1422,30 +1404,12 @@ impl Grid {
         let operator = operator.as_standard_layout();
 
         // determine what and how many hadrons are in the initial state
-        let initial_state_1 = self
-            .key_values()
-            .map_or(Some("2212"), |kv| {
-                kv.get("initial_state_1").map(String::as_str)
-            })
-            .map(str::parse::<i32>)?
-            .ok()?;
-        let initial_state_2 = self
-            .key_values()
-            .map_or(Some("2212"), |kv| {
-                kv.get("initial_state_2").map(String::as_str)
-            })
-            .map(str::parse::<i32>)?
-            .ok()?;
+        let initial_state_1 = self.initial_state_1();
+        let initial_state_2 = self.initial_state_2();
 
         // are the initial states hadrons?
-        let has_pdf1 = !self
-            .lumi()
-            .iter()
-            .all(|entry| entry.entry().iter().all(|&(a, _, _)| a == initial_state_1));
-        let has_pdf2 = !self
-            .lumi()
-            .iter()
-            .all(|entry| entry.entry().iter().all(|&(_, b, _)| b == initial_state_2));
+        let has_pdf1 = self.has_pdf1();
+        let has_pdf2 = self.has_pdf2();
 
         let pids1 = if has_pdf1 {
             eko_info.grid_axes.pids.clone()
@@ -1815,6 +1779,59 @@ impl Grid {
         FkTable::try_from(result).ok()
     }
 
+    /// Converts this `Grid` into an [`FkTable`] using an evolution kernel operator (EKO) given as
+    /// `operator`. The dimensions and properties of this operator must be described using `info`.
+    /// The parameter `order_mask` can be used to include or exclude orders from this operation,
+    /// and must correspond to the ordering given by [`Grid::orders`]. Orders that are not given
+    /// are enabled, and in particular if `order_mask` is empty all orders are activated.
+    ///
+    /// # Errors
+    ///
+    /// Returns a [`GridError::EvolutionFailure`] if either the `operator` or its `info` is
+    /// incompatible with this `Grid`.
+    pub fn evolve(
+        &self,
+        operator: &Array5<f64>,
+        info: &OperatorInfo,
+        order_mask: &[bool],
+    ) -> Result<FkTable, GridError> {
+        let op_info_dim = (
+            info.fac1.len(),
+            info.pids1.len(),
+            info.x1.len(),
+            info.pids0.len(),
+            info.x0.len(),
+        );
+
+        if operator.dim() != op_info_dim {
+            return Err(GridError::EvolutionFailure(format!(
+                "operator information {:?} does not match the operator's dimensions: {:?}",
+                op_info_dim,
+                operator.dim(),
+            )));
+        }
+
+        let (subgrids, lumi) = if self.has_pdf1() && self.has_pdf2() {
+            evolution::evolve_with_two(self, operator, info, order_mask)
+        } else {
+            evolution::evolve_with_one(self, operator, info, order_mask)
+        }?;
+
+        let mut grid = Self {
+            subgrids,
+            lumi,
+            bin_limits: self.bin_limits.clone(),
+            orders: vec![Order::new(0, 0, 0, 0)],
+            subgrid_params: SubgridParams::default(),
+            more_members: self.more_members.clone(),
+        };
+
+        // write additional metadata
+        grid.set_key_value("lumi_id_types", &info.lumi_id_types);
+
+        Ok(FkTable::try_from(grid).unwrap())
+    }
+
     /// Deletes bins with the corresponding `bin_indices`. Repeated indices and indices larger or
     /// equal the bin length are ignored.
     pub fn delete_bins(&mut self, bin_indices: &[usize]) {
@@ -1925,6 +1942,54 @@ impl Grid {
             })
             .collect();
     }
+
+    /// Returns `true` if the first initial state needs a convolution, `false` otherwise.
+    #[must_use]
+    pub fn has_pdf1(&self) -> bool {
+        let initial_state_1 = self.initial_state_1();
+
+        !self
+            .lumi()
+            .iter()
+            .all(|entry| entry.entry().iter().all(|&(a, _, _)| a == initial_state_1))
+    }
+
+    /// Returns `true` if the second initial state needs a convolution, `false` otherwise.
+    #[must_use]
+    pub fn has_pdf2(&self) -> bool {
+        let initial_state_2 = self.initial_state_2();
+
+        !self
+            .lumi()
+            .iter()
+            .all(|entry| entry.entry().iter().all(|&(_, b, _)| b == initial_state_2))
+    }
+
+    /// Returns the particle identifier of the first initial state. This is usually but not always
+    /// a proton, which is represented by the PDG ID `2212`.
+    #[must_use]
+    pub fn initial_state_1(&self) -> i32 {
+        self.key_values()
+            .map_or(Some("2212"), |kv| {
+                kv.get("initial_state_1").map(String::as_str)
+            })
+            .map(str::parse)
+            .unwrap()
+            .unwrap()
+    }
+
+    /// Returns the particle identifier of the second initial state. This is usually but not always
+    /// a proton, which is represented by the PDG ID `2212`.
+    #[must_use]
+    pub fn initial_state_2(&self) -> i32 {
+        self.key_values()
+            .map_or(Some("2212"), |kv| {
+                kv.get("initial_state_2").map(String::as_str)
+            })
+            .map(str::parse)
+            .unwrap()
+            .unwrap()
+    }
 }
 
 #[cfg(test)]
