@@ -168,67 +168,6 @@ impl Subgrid for ImportOnlySubgridV1 {
     }
 }
 
-impl From<&LagrangeSubgridV2> for ImportOnlySubgridV1 {
-    fn from(subgrid: &LagrangeSubgridV2) -> Self {
-        let array = subgrid.grid.as_ref().map_or_else(
-            || SparseArray3::new(subgrid.ntau, subgrid.ny1, subgrid.ny2),
-            // in the following case we should optimize when ny2 > ny1
-            |array| {
-                let reweight_x1: Vec<_> = subgrid
-                    .x1_grid()
-                    .iter()
-                    .map(|x| lagrange_subgrid::weightfun(*x))
-                    .collect();
-                let reweight_x2: Vec<_> = subgrid
-                    .x2_grid()
-                    .iter()
-                    .map(|x| lagrange_subgrid::weightfun(*x))
-                    .collect();
-
-                if subgrid.static_q2 > 0.0 {
-                    // in this case we've detected a static scale for this bin and we can collapse
-                    // the Q^2 axis into a single bin
-
-                    let mut array = array
-                        .sum_axis(Axis(0))
-                        .into_shape((1, subgrid.ny1, subgrid.ny2))
-                        .unwrap();
-                    for ((_, ix1, ix2), entry) in array.indexed_iter_mut() {
-                        *entry *= reweight_x1[ix1] * reweight_x2[ix2];
-                    }
-                    SparseArray3::from_ndarray(&array, 0, 1)
-                } else {
-                    let mut array = array.clone();
-                    for ((_, ix1, ix2), entry) in array.indexed_iter_mut() {
-                        *entry *= reweight_x1[ix1] * reweight_x2[ix2];
-                    }
-                    SparseArray3::from_ndarray(&array, 0, subgrid.itaumax - subgrid.itaumin)
-                }
-            },
-        );
-        let q2_grid = if subgrid.static_q2 > 0.0 {
-            vec![subgrid.static_q2]
-        } else {
-            subgrid
-                .mu2_grid()
-                .iter()
-                .skip(subgrid.itaumin)
-                .take(subgrid.itaumax - subgrid.itaumin)
-                .map(|mu2| mu2.fac)
-                .collect()
-        };
-        let x1_grid = subgrid.x1_grid().into_owned();
-        let x2_grid = subgrid.x2_grid().into_owned();
-
-        Self {
-            array,
-            q2_grid,
-            x1_grid,
-            x2_grid,
-        }
-    }
-}
-
 /// TODO
 #[derive(Clone, Deserialize, Serialize)]
 pub struct ImportOnlySubgridV2 {
@@ -300,9 +239,38 @@ impl Subgrid for ImportOnlySubgridV2 {
             if self.array.is_empty() && !transpose {
                 mem::swap(&mut self.array, &mut other_grid.array);
             } else {
-                // TODO: the general case isn't implemented
-                assert!(self.x1_grid() == other_grid.x1_grid());
-                assert!(self.x2_grid() == other_grid.x2_grid());
+                if self.x1_grid() != other_grid.x1_grid() || self.x2_grid() != other_grid.x2_grid()
+                {
+                    let mut x1_grid = self.x1_grid.to_owned();
+                    let mut x2_grid = self.x2_grid.to_owned();
+
+                    x1_grid.extend_from_slice(&other_grid.x1_grid());
+                    x1_grid.sort_by(|a, b| a.partial_cmp(b).unwrap());
+                    x1_grid.dedup();
+                    x2_grid.extend_from_slice(&other_grid.x2_grid());
+                    x2_grid.sort_by(|a, b| a.partial_cmp(b).unwrap());
+                    x2_grid.dedup();
+
+                    let mut array =
+                        SparseArray3::new(self.array.dimensions().0, x1_grid.len(), x2_grid.len());
+
+                    for ((i, j, k), value) in self.array.indexed_iter() {
+                        let target_j = x1_grid
+                            .iter()
+                            .position(|&x| x == self.x1_grid[j])
+                            .unwrap_or_else(|| unreachable!());
+                        let target_k = x2_grid
+                            .iter()
+                            .position(|&x| x == self.x2_grid[k])
+                            .unwrap_or_else(|| unreachable!());
+
+                        array[[i, target_j, target_k]] = *value;
+                    }
+
+                    self.array = array;
+                    self.x1_grid = x1_grid;
+                    self.x2_grid = x2_grid;
+                }
 
                 for (other_index, mu2) in other_grid.mu2_grid().iter().enumerate() {
                     let index = match self
@@ -380,11 +348,81 @@ impl Subgrid for ImportOnlySubgridV2 {
     }
 }
 
+impl From<&LagrangeSubgridV2> for ImportOnlySubgridV2 {
+    fn from(subgrid: &LagrangeSubgridV2) -> Self {
+        let array = subgrid.grid.as_ref().map_or_else(
+            || SparseArray3::new(subgrid.ntau, subgrid.ny1, subgrid.ny2),
+            // in the following case we should optimize when ny2 > ny1
+            |array| {
+                let reweight_x1: Vec<_> = subgrid
+                    .x1_grid()
+                    .iter()
+                    .map(|x| lagrange_subgrid::weightfun(*x))
+                    .collect();
+                let reweight_x2: Vec<_> = subgrid
+                    .x2_grid()
+                    .iter()
+                    .map(|x| lagrange_subgrid::weightfun(*x))
+                    .collect();
+
+                if subgrid.static_q2 > 0.0 {
+                    // in this case we've detected a static scale for this bin and we can collapse
+                    // the Q^2 axis into a single bin
+
+                    let mut array = array
+                        .sum_axis(Axis(0))
+                        .into_shape((1, subgrid.ny1, subgrid.ny2))
+                        .unwrap();
+                    for ((_, ix1, ix2), entry) in array.indexed_iter_mut() {
+                        *entry *= reweight_x1[ix1] * reweight_x2[ix2];
+                    }
+                    SparseArray3::from_ndarray(&array, 0, 1)
+                } else {
+                    let mut array = array.clone();
+                    for ((_, ix1, ix2), entry) in array.indexed_iter_mut() {
+                        *entry *= reweight_x1[ix1] * reweight_x2[ix2];
+                    }
+                    SparseArray3::from_ndarray(&array, 0, subgrid.itaumax - subgrid.itaumin)
+                }
+            },
+        );
+        let mu2_grid = if subgrid.static_q2 > 0.0 {
+            vec![Mu2 {
+                ren: subgrid.static_q2,
+                fac: subgrid.static_q2,
+            }]
+        } else {
+            subgrid
+                .mu2_grid()
+                .iter()
+                .skip(subgrid.itaumin)
+                .take(subgrid.itaumax - subgrid.itaumin)
+                .map(|mu2| Mu2 {
+                    ren: mu2.ren,
+                    fac: mu2.fac,
+                })
+                .collect()
+        };
+        let x1_grid = subgrid.x1_grid().into_owned();
+        let x2_grid = subgrid.x2_grid().into_owned();
+
+        Self {
+            array,
+            mu2_grid,
+            x1_grid,
+            x2_grid,
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
     use crate::subgrid::{ExtraSubgridParams, SubgridParams};
     use float_cmp::assert_approx_eq;
+    use rand::distributions::{Distribution, Uniform};
+    use rand::Rng;
+    use rand_pcg::Pcg64;
 
     #[test]
     fn test_v1() {
@@ -664,7 +702,7 @@ mod tests {
         let lumi = &mut (|_, _, _| 1.0) as &mut dyn FnMut(usize, usize, usize) -> f64;
         let reference = lagrange.convolute(&x1, &x2, &mu2, lumi);
 
-        let imported = ImportOnlySubgridV1::from(&lagrange);
+        let imported = ImportOnlySubgridV2::from(&lagrange);
         let test = imported.convolute(&x1, &x2, &mu2, lumi);
 
         // make sure the conversion did not change the results
@@ -675,4 +713,54 @@ mod tests {
         // points as its interpolation order
         assert_eq!(imported.mu2_grid().len(), 4);
     }
+
+    #[test]
+    fn merge_with_different_x_grids() {
+        let mut params = SubgridParams::default();
+        let mut grid1 = LagrangeSubgridV2::new(&params, &ExtraSubgridParams::default());
+
+        // change parameters of the second grid to force non-trivial merging
+        params.set_x_min(0.2);
+        params.set_x_max(0.5);
+
+        let mut grid2 = LagrangeSubgridV2::new(&params, &ExtraSubgridParams::default());
+        let mut rng = Pcg64::new(0xcafef00dd15ea5e5, 0xa02bdbf7bb3c0a7ac28fa16a64abf96);
+        let q2_range = Uniform::new(1e4, 1e8);
+
+        for _ in 0..1000 {
+            grid1.fill(&Ntuple {
+                x1: rng.gen(),
+                x2: rng.gen(),
+                q2: q2_range.sample(&mut rng),
+                weight: 1.0,
+            });
+            grid2.fill(&Ntuple {
+                x1: rng.gen(),
+                x2: rng.gen(),
+                q2: q2_range.sample(&mut rng),
+                weight: 1.0,
+            });
+        }
+
+        let lumi = &mut (|_, _, _| 1.0) as &mut dyn FnMut(usize, usize, usize) -> f64;
+        let result1 = grid1.convolute(&grid1.x1_grid(), &grid1.x2_grid(), &grid1.mu2_grid(), lumi);
+        let result2 = grid2.convolute(&grid2.x1_grid(), &grid2.x2_grid(), &grid2.mu2_grid(), lumi);
+
+        let mut grid1: SubgridEnum = ImportOnlySubgridV2::from(&grid1).into();
+        let mut grid2: SubgridEnum = ImportOnlySubgridV2::from(&grid2).into();
+
+        let result3 = grid1.convolute(&grid1.x1_grid(), &grid1.x2_grid(), &grid1.mu2_grid(), lumi);
+        let result4 = grid2.convolute(&grid2.x1_grid(), &grid2.x2_grid(), &grid2.mu2_grid(), lumi);
+
+        // conversion from LangrangeSubgridV2 to ImportOnlySubgridV2 shouldn't change the results
+        assert!((result3 / result1 - 1.0).abs() < 1e-13);
+        assert!((result4 / result2 - 1.0).abs() < 1e-13);
+
+        grid1.merge(&mut grid2, false);
+
+        let result5 = grid1.convolute(&grid1.x1_grid(), &grid1.x2_grid(), &grid1.mu2_grid(), lumi);
+
+        // merging the two grids should give the sum of the two results
+        assert!((result5 / (result3 + result4) - 1.0).abs() < 1e-12);
+    }
 }
