@@ -1,3 +1,4 @@
+use pineappl::evolution::OperatorInfo;
 use pineappl::grid::{EkoInfo, Grid, GridAxes, Ntuple, Order};
 use pineappl::lumi::LumiCache;
 
@@ -458,6 +459,81 @@ impl PyGrid {
         }
     }
 
+    /// Convolute with grid with an evolution operator.
+    ///
+    /// Parameters
+    /// ----------
+    /// operator : numpy.ndarray(int, rank=5)
+    ///     evolution tensor
+    /// fac0 : float
+    ///     reference scale
+    /// pids0 : numpy.ndarray(int)
+    ///     sorting of the particles in the tensor for final FkTable
+    /// x0 : numpy.ndarray(float)
+    ///     final FKTable interpolation grid
+    /// fac1 : numpy.ndarray(float)
+    ///     list of factorization scales
+    /// pids1 : numpy.ndarray(int)
+    ///     sorting of the particles in the grid
+    /// x1 : numpy.ndarray(float)
+    ///     interpolation grid at process level
+    /// ren1 : numpy.ndarray(float)
+    ///     list of renormalization scales
+    /// alphas : numpy.ndarray(float)
+    ///     list with :math:`\alpha_s(Q2)` for the process scales
+    /// xi : (float, float)
+    ///     factorization and renormalization variation
+    /// lumi_id_types : str
+    ///     type of luminosity identifier
+    /// order_mask : numpy.ndarray(bool)
+    ///     boolean mask to activate orders
+    ///
+    /// Returns
+    /// -------
+    /// PyFkTable :
+    ///     produced FK table
+    pub fn evolve(
+        &self,
+        operator: PyReadonlyArray5<f64>,
+        fac0: f64,
+        pids0: PyReadonlyArray1<i32>,
+        x0: PyReadonlyArray1<f64>,
+        fac1: PyReadonlyArray1<f64>,
+        pids1: PyReadonlyArray1<i32>,
+        x1: PyReadonlyArray1<f64>,
+        ren1: PyReadonlyArray1<f64>,
+        alphas: PyReadonlyArray1<f64>,
+        xi: (f64, f64),
+        lumi_id_types: String,
+        order_mask: PyReadonlyArray1<bool>,
+    ) -> PyFkTable {
+        let op_info = OperatorInfo {
+            fac0: fac0,
+            pids0: pids0.to_vec().unwrap(),
+            x0: x0.to_vec().unwrap(),
+            fac1: fac1.to_vec().unwrap(),
+            pids1: pids1.to_vec().unwrap(),
+            x1: x1.to_vec().unwrap(),
+            ren1: ren1.to_vec().unwrap(),
+            alphas: alphas.to_vec().unwrap(),
+            xir: xi.0,
+            xif: xi.1,
+            lumi_id_types: lumi_id_types,
+        };
+
+        let evolved_grid = self
+            .grid
+            .evolve(
+                &operator.as_array().to_owned(),
+                &op_info,
+                &order_mask.to_vec().unwrap(),
+            )
+            .expect("Nothing returned from evolution.");
+        PyFkTable {
+            fk_table: evolved_grid,
+        }
+    }
+
     /// Load grid from file.
     ///
     /// **Usage:** `pineko`, FKTable generation
